﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using PeteBrown.SilverlightSoundTest.Synth;
using PeteBrown.SilverlightSoundTest.SoundRenderer;
using PeteBrown.SilverlightSoundTest.Synth.Wave;
using PeteBrown.SilverlightSoundTest.Synth.Oscillators;

namespace PeteBrown.SilverlightSoundTest
{
    public partial class Page : UserControl
    {
        private Synthesizer _synth;

        public Page()
        {
            InitializeComponent();

            BuildSynthsizer();

            //AudioPlayer.BufferingTime = TimeSpan.FromSeconds(.1);
            //AudioPlayer.AutoPlay = true;
            //AudioPlayer.MediaFailed += new EventHandler<ExceptionRoutedEventArgs>(AudioPlayer_MediaFailed);
            //AudioPlayer.MediaOpened += new RoutedEventHandler(AudioPlayer_MediaOpened);
            Loaded += new RoutedEventHandler(Page_Loaded);

            ToggleArpeggiator.Click += new RoutedEventHandler(ToggleArpeggiator_Click);

        }

        void ToggleArpeggiator_Click(object sender, RoutedEventArgs e)
        {
            if (_synth.Arpeggiator.IsRunning)
            {
                _synth.Arpeggiator.Stop();
            }
            else
            {
                // TODO turn off currently playing notes, but track which ones they are and build the arpeggio from them

                _synth.Arpeggiator.Pattern = ArpeggiatorPattern.DecreasingIncreasing;

                _synth.Arpeggiator.Notes.Clear();

                // Temp notes

                _synth.Arpeggiator.Notes.Add(new Note(Notes.C, 3));
                _synth.Arpeggiator.Notes.Add(new Note(Notes.F, 3));
                _synth.Arpeggiator.Notes.Add(new Note(Notes.A, 4));
                _synth.Arpeggiator.Notes.Add(new Note(Notes.C, 4));
                _synth.Arpeggiator.Notes.Add(new Note(Notes.F, 4));
                _synth.Arpeggiator.Notes.Add(new Note(Notes.A, 5));
                _synth.Arpeggiator.Notes.Add(new Note(Notes.C, 5));

                _synth.Arpeggiator.Start();
            }
        }



        //void AudioPlayer_MediaOpened(object sender, RoutedEventArgs e)
        //{
        //    AudioPlayer.Play();
        //}

        //void AudioPlayer_MediaFailed(object sender, ExceptionRoutedEventArgs e)
        //{
            
        //}

        void Page_Loaded(object sender, RoutedEventArgs e)
        {
            BindUI();

            AudioPlayer.SetSource(_synth);
        }

        private void BindUI()
        {
            this.Keyboard.Synth = _synth;
            this.MixerEditor.Synth = _synth;

            Oscillator1Editor.Synth = _synth;
            Oscillator1Editor.OscillatorParameters = _synth.GetOscillatorParameters(0);

            Oscillator2Editor.Synth = _synth;
            Oscillator2Editor.OscillatorParameters = _synth.GetOscillatorParameters(1);

            Oscillator3Editor.Synth = _synth;
            Oscillator3Editor.OscillatorParameters = _synth.GetOscillatorParameters(2);

        }

        private void BuildSynthsizer()
        {

            _synth = new Synthesizer(32);

            OscillatorParameters osc1 = _synth.AddOscillator();
            osc1.Waveform = _synth.CreateTriangleWaveform();
            osc1.Title = "Oscillator 1";
            osc1.IsEnabled = true;
            osc1.PhaseOffset = new Percent(0);

            OscillatorParameters osc2 = _synth.AddOscillator();
            osc2.Detune.Offset = -1;
            osc2.Waveform = _synth.CreateTriangleWaveform();
            osc2.Pan.Position = -0.5;
            osc2.Title = "Oscillator 2";
            osc2.IsEnabled = true;
            osc2.PhaseOffset = new Percent(.16);
            osc2.Polarity = Polarity.Positive;

            OscillatorParameters osc3 = _synth.AddOscillator();
            osc3.Detune.Offset = +1;
            osc3.Waveform = _synth.CreateTriangleWaveform();
            osc3.Pan.Position = +0.5;
            osc3.Title = "Oscillator 3";
            osc3.IsEnabled = true;
            osc3.PhaseOffset = new Percent(.28);
            osc3.Polarity = Polarity.Negative;

            _synth.Volume = 1.0;

        }


    }
}
