﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.Windows.Threading;

namespace PeteBrown.SilverlightSoundTest.Synth
{
    public enum ArpeggiatorPattern
    {
        Increasing,
        Decreasing,
        IncreasingDecreasing,
        DecreasingIncreasing,
        Random
    }

    public class Arpeggiator
    {
        private Random _random;

        private TimeSpan _noteHold = TimeSpan.FromSeconds(0.12);
        //private TimeSpan _rest = TimeSpan.FromSeconds(0.1);
        private Synthesizer _synth;

        private List<Note> _notes = new List<Note>();  // order of notes in this list is significant

        private int _currentPatternPosition = 0;
        private int _currentDirection = 1;          // -1 or 1 to handle increasing or decreasing
        private Voice _currentVoice;

        private ArpeggiatorPattern _pattern = ArpeggiatorPattern.Increasing;

        private DispatcherTimer _timer = new DispatcherTimer();

        private bool _running;

        public Arpeggiator(Synthesizer synth)
        {
            _synth = synth;
            _random = new Random();

        }

        public void Start()
        {
            switch (_pattern)
            {
                case ArpeggiatorPattern.Decreasing:
                case ArpeggiatorPattern.DecreasingIncreasing:
                    _currentPatternPosition = _notes.Count-1;
                    _currentDirection = -1;
                    break;

                case ArpeggiatorPattern.Increasing:
                case ArpeggiatorPattern.IncreasingDecreasing:
                    _currentPatternPosition = 0;
                    _currentDirection = 1;
                    break;

                case ArpeggiatorPattern.Random:
                    break;
            }

            _running = true;

            _timer.Tick += new EventHandler(_timer_Tick);
            _timer.Interval = _noteHold;
            _timer.Start();
        }

        // TODO: Need to do some checking to make sure there are valid notes in the pattern
        // TODO: Need to implement the other patterns. Currently only handles ascending

        void _timer_Tick(object sender, EventArgs e)
        {
            if (_currentVoice != null)
                _currentVoice.Release();


            switch (_pattern)
            {
                case ArpeggiatorPattern.Decreasing:
                    _currentPatternPosition -= 1;

                    if (_currentPatternPosition < 0)
                        _currentPatternPosition = _notes.Count -1;
                    break;
                
                case ArpeggiatorPattern.Increasing:
                    _currentPatternPosition += 1;

                    if (_currentPatternPosition == _notes.Count)
                        _currentPatternPosition = 0;
                    break;

                case ArpeggiatorPattern.DecreasingIncreasing:
                case ArpeggiatorPattern.IncreasingDecreasing:

                    // TODO: Need to prevent re-triggering of outside notes

                    _currentPatternPosition += _currentDirection;

                    if (_currentPatternPosition == _notes.Count || _currentPatternPosition < 0)
                    {
                        _currentDirection = -1 * _currentDirection;

                        _currentPatternPosition += (_currentDirection * 2);
                    }

                    break;

                case ArpeggiatorPattern.Random:
                    _currentPatternPosition = _random.Next(0, _notes.Count-1);
                    break;
            }

            _currentVoice = _synth.TriggerNote(_notes[_currentPatternPosition]);

        }

        public void Stop()
        {
            _timer.Stop();

            if (_currentVoice != null)
                _currentVoice.Release();

            _running = false;
        }

        public List<Note> Notes
        {
            get { return _notes; }
            set { _notes = value; }
        }

        public bool IsRunning
        {
            get { return _running; }
        }

        public ArpeggiatorPattern Pattern
        {
            get { return _pattern; }
            set
            { 
                _pattern = value;

            }
        } 

    }
}
