﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using PeteBrown.SilverlightSoundTest.Synth.Utilities;

namespace PeteBrown.SilverlightSoundTest.Synth.Envelopes
{
    // note, may want to consider ADSHR and AHDSR or even AHDSHR where H is a hold
    // ref: http://en.wikipedia.org/wiki/Envelope_generator
    // ref: http://en.wikipedia.org/wiki/ADSR_envelope

    // TODO: Allow for a bezier curve to the attack, more than just a straight rise over time
    public class AmplitudeEnvelope
    {
        private EnvelopeParameters _parameters;

        public void Initialize(EnvelopeParameters parameters)
        {
            _parameters = parameters;
        }

        //public void Initialize(SampleRate sampleRate /* , Range<TimeSpan> attackRange, Range<TimeSpan> decayRange, Range<TimeSpan> releaseRange */)
        //{
        //    _sampleRate = sampleRate;

        //    //AttackRange = attackRange;
        //    //DecayRange = decayRange;
        //    ////SustainRange = new Range<Amplitude>(new Amplitude(...));
        //    //ReleaseRange = releaseRange;

        //    //Attack = TimeSpan.FromSeconds(0.05);
        //    //Decay = TimeSpan.FromSeconds(0.01);
        //    //Sustain = new Percent(0.80);
        //    //Release = TimeSpan.FromSeconds(0.25);

        //    Attack = 0.05;
        //    Decay = 0.0;
        //    Sustain = new Percent(0.75);
        //    Release = 0.5;

        //}

        // TODO: Need to allow for keytracking/scaling
        // TODO: Need to account for release during attack or decay, and release at constant slope http://www.kvraudio.com/forum/viewtopic.php?t=237285
        public void Output(SoundBuffer<MonoSample> inputBuffer, SoundBuffer<MonoSample> outputBuffer, EnvelopeState state, int offset, int count)
        {
            for (int i = offset; i < count; i++)
            {
                double maxAmplitude = 1.0;      // %
                double amplitudeModifier = 1.0;

                switch (state.State)
                {
                    case AdsrEnvelopeState.Attack:
                        //amplitudeModifier = MathUtility.GetYValueOnLineFromOrigin((double)Attack.Ticks, 1.0, (double)state.ElapsedTimeAtCurrentState.Ticks);
                        amplitudeModifier = MathUtility.GetYValueOnLineFromOrigin(_parameters.Attack, maxAmplitude, state.ElapsedSecondsAtCurrentState);
                        break;

                    case AdsrEnvelopeState.Decay:
                        amplitudeModifier = MathUtility.GetYValueOnLine(0.0, maxAmplitude, _parameters.Decay, _parameters.Sustain.Value, maxAmplitude, state.ElapsedSecondsAtCurrentState);
                        break;

                    case AdsrEnvelopeState.Sustain:
                        amplitudeModifier = _parameters.Sustain.Value;
                        break;

                    case AdsrEnvelopeState.Release:
                        // TODO: Need to track current max volume in case we go from attack to release or decay to release
                        // and pass that in where I have Sustain.Value below.
                        amplitudeModifier = MathUtility.GetYValueOnLine(0.0, _parameters.Sustain.Value, _parameters.Release, 0.0, _parameters.Sustain.Value, state.ElapsedSecondsAtCurrentState);
                        break;

                    case AdsrEnvelopeState.Completed:
                        amplitudeModifier = 0.0;
                        break;
                }

                outputBuffer[i].Value = (int)(inputBuffer[i].Value * amplitudeModifier);



                // increment timer. Note, will ned to allow for keytracking / scaling and modify this 
                state.IncrementElapsedTime(_parameters.SampleRate.SecondsPerSample);

                if (state.ElapsedSecondsAtCurrentState > _parameters.Attack && state.State == AdsrEnvelopeState.Attack)
                    state.State = AdsrEnvelopeState.Decay;
                else if (state.ElapsedSecondsAtCurrentState > _parameters.Decay && state.State == AdsrEnvelopeState.Decay)
                    state.State = AdsrEnvelopeState.Sustain;
                else if (state.ElapsedSecondsAtCurrentState > _parameters.Release && state.State == AdsrEnvelopeState.Release)
                    state.State = AdsrEnvelopeState.Completed;


            }
        }

        public EnvelopeState GetNewState()
        {
            return new EnvelopeState();
        }

        //public void Ouput(SoundBuffer<StereoSample> inputBuffer, SoundBuffer<StereoSample> outputBuffer, EnvelopeState state, long offset, int count)
        //{
        //}




    }
}
