﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;

namespace PeteBrown.SilverlightSoundTest.Synth.Envelopes
{
    public class EnvelopeParameters : INotifyPropertyChanged
    {

        private double _attack;             // time period in seconds
        private double _decay;              // time period in seconds
        private Percent _sustain;           // volume level expressed as a number from 0..1
        private double _release;            // time period in seconds


        public double MaxAttack { get; set; }
        public double MinAttack { get; set; }

        public double MaxDecay { get; set; }
        public double MinDecay { get; set; }

        public double MaxRelease { get; set; }
        public double MinRelease { get; set; }

        private SampleRate _sampleRate;

        //public Percent Attack { get; set; }          // time period - % of max
        //public Percent Decay { get; set; }           // time period - % of max
        //public Percent Sustain { get; set; }         // volume level expressed as a number from 0..1
        //public Percent Release { get; set; }         // time period

        //public Range<TimeSpan> AttackRange { get; set; }
        //public Range<TimeSpan> DecayRange { get; set; }
        ////public Range<Amplitude> SustainRange { get; set; }
        //public Range<TimeSpan> ReleaseRange { get; set; }

        //public bool Inverted { get; set; }

        public EnvelopeParameters()
        {
            MaxAttack = 2;
            MinAttack = 0;

            MaxDecay = 2;
            MinDecay = 0;

            MaxRelease = 2;
            MinRelease = 0;


            _attack = 0.05;
            _decay = 0.15;
            _sustain = new Percent(1);
            _release = 0.50;
        }

        public void Initialize(SampleRate sampleRate)
        {
            _sampleRate = sampleRate;
        }

        public SampleRate SampleRate
        {
            get { return _sampleRate; }
        }


        public double Attack
        {
            get { return _attack; }
            set { _attack = value; NotifyChanged("Attack"); }
        }

        public double Decay
        {
            get { return _decay; }
            set { _decay = value; NotifyChanged("Decay"); }
        }

        public Percent Sustain
        {
            get { return _sustain; }
            set { _sustain = value; NotifyChanged("Sustain"); }
        }

        public double Release
        {
            get { return _release; }
            set { _release = value; NotifyChanged("Release"); }
        }



        #region INotifyPropertyChanged Members

        private void NotifyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }


        public event PropertyChangedEventHandler PropertyChanged;


        #endregion




    }
}
