﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;

namespace PeteBrown.SilverlightSoundTest.Synth
{
    public class FrequencyOffset : INotifyPropertyChanged
    {
        private double _minOffset;
        private double _maxOffset;
        private double _offset;

        public FrequencyOffset(double minOffset, double maxOffset, double currentOffset)
        {
            _minOffset = minOffset;
            _maxOffset = maxOffset;
            _offset = currentOffset;

            NotifyChanged("MinOffset");
            NotifyChanged("MaxOffset");
            NotifyChanged("Offset");
        }


        public double MinOffset
        {
            get { return _minOffset; }
        }

        public double MaxOffset
        {
            get { return _maxOffset; }
        }

        public double Offset
        {
            get { return _offset; }
            set { _offset = ScrubOffset(value); NotifyChanged("Offset"); }
        }

        private double ScrubOffset(double value)
        {
            if (value < _minOffset) return _minOffset;
            if (value > _maxOffset) return _maxOffset;

            return value;
        }

        #region INotifyPropertyChanged Members

        private void NotifyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }


        public event PropertyChangedEventHandler PropertyChanged;


        #endregion

    }
}
