﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using PeteBrown.SilverlightSoundTest.Synth.Utilities;
using System.Diagnostics;

// useful reference: http://www.sengpielaudio.com/calculator-notenames.htm

namespace PeteBrown.SilverlightSoundTest.Synth
{
    public enum Notes
    {
        C = 0,
        CSharp = 1,
        DFlat = 1,
        D = 2,
        DSharp = 3,
        EFlat = 3,
        E = 4,
        F = 5,
        FSharp = 6,
        GFlat = 6,
        G = 7,
        GSharp = 8,
        AFlat = 8,
        A = 9,
        ASharp = 10,
        BFlat = 10,
        B = 11
    }



    public class Note
    {
        public const int NotesPerOctave = 12;
        public const int NoteCount = 128;

        public const int A440NoteNumber = 69;
        public const float A440Frequency = 440;       // 440hz
        public const int MiddleCNoteNumber = 60;    // per MIDI specification

        private int _number;
        private float _frequency;

        public static float GetFrequency(int noteNumber)
        {
            float val = PowerOfTwoTable.GetPower((float)(noteNumber - A440NoteNumber) / NotesPerOctave) * A440Frequency;

            return val;
        }

        #region Constructors

        public Note()
        {
            Number = A440NoteNumber;
        }

        // uses Midi note numbers C(-1) is note 0, G9 is note 127
        public Note(int number)
        {
            Debug.Assert(number >= 0 && number < NoteCount);

            Number = number;
        }

        public Note(Notes name, int octave)
        {
            Debug.Assert(octave >= -1 && octave <= 9);

            if (octave == 9)
            {
            //    Debug.Assert(! NoteName in {NoteName.GSharp, NoteName.AFlat, NoteName.A, NoteName.ASharp, NoteName.BFlat, NoteName.B})
            }

            // convert to number
            Number = (octave * NotesPerOctave) + (int)name;

        }

        #endregion


        public float Frequency
        {
            get
            {
                return _frequency;
            }
        }

        public int Number
        {
            get
            {
                return _number;
            }

            set
            {
                _number = value;

                _frequency = GetFrequency((int)_number);
            }
        }


        public override string ToString()
        {
            return string.Format("{0}", _number);
        }

    }
}
