﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using PeteBrown.SilverlightSoundTest.Synth.Oscillators;
using PeteBrown.SilverlightSoundTest.Synth.Wave;
using System.Diagnostics;
using PeteBrown.SilverlightSoundTest.Synth.Filters;
using System.ComponentModel;
using PeteBrown.SilverlightSoundTest.Synth.Envelopes;


namespace PeteBrown.SilverlightSoundTest.Synth
{
    public class Oscillator : IOscillator
    {
        //private IFilter _filter;                                // filter that processes sound after waveform generates it
        private WaveformMap _waveMap;                           // TODO
        private WaveTable _waveTable;                           // TODO

        //private double _detuneDeltaFrequency = 0.0;             // amount to detune this oscillator. frequency

        private Note _currentNote;                              // note we're playing, not taking transpose or detune into account
        private bool _isTriggered;                              // true if this oscillator has a note triggered.
        private bool _isGeneratingSound;                        // true if there's any sound coming from this oscillator. Takes ADSR into account

        SoundBuffer<MonoSample> _buffer;                        // buffer to which this oscillator's sound is written when requested

        private OscillatorParameters _parameters;               // common parameters across all instances of this oscillator
            
        private WaveformState _waveformState;                   // current state for the waveform we're using

        // TEMP
        private AmplitudeEnvelope _envelope = new AmplitudeEnvelope();
        private EnvelopeState _envelopeState;


        public void Initialize(SoundBuffer<MonoSample> buffer, OscillatorParameters parameters)
        {
            _parameters = parameters;

            _buffer = buffer;

            _envelope.Initialize(parameters.AmplitudeEnvelopeParameters);
            _envelopeState = _envelope.GetNewState();
        }



        // hack
        private void SynchronizeWaveformStateWithSettings(WaveformState state)
        {
            state.Polarity = _parameters.Polarity;
            state.PhaseOffset = _parameters.PhaseOffset;
        }



        public SoundBuffer<MonoSample> OutputOneCompleteCycle(Note note)
        {
            int sampleCount = _parameters.Waveform.GetSampleCount(note.Frequency);

            SoundBuffer<MonoSample> buffer = new SoundBuffer<MonoSample>(sampleCount);

            WaveformState state = _parameters.Waveform.GetNewState();
            SynchronizeWaveformStateWithSettings(state);

            state.MaxAmplitude = new Amplitude(int.MaxValue);

            for (int i = 0; i < sampleCount; i++)
            {
                buffer[i] = _parameters.Waveform.GetSample(note.Frequency, state);
            }

            return buffer;

        }


        public void Output(long offset, int count)
        {
         //   System.Diagnostics.Debug.WriteLine("Oscillator.Output");

            if (_parameters.IsEnabled)
            {
                if (_isGeneratingSound)
                {
                    if (_waveformState == null)
                        _waveformState = _parameters.Waveform.GetNewState();

                    SynchronizeWaveformStateWithSettings(_waveformState);

                    if (_currentNote != null)
                    {
                        // calculate the final frequency for this oscillator
                        double frequency = _parameters.Transpose.GetFrequency(_currentNote) + _parameters.Detune.Offset;
                        for (int i = 0; i < count; i++)
                        {
                            _buffer[i] = _parameters.Waveform.GetSample(frequency, _waveformState);
                        }

                    }
                    else
                    {
                        for (int i = 0; i < count; i++)
                        {
                            _buffer[i] = null;
                        }
                    }


                    // TEMP!
                    _envelope.Output(_buffer, _buffer, _envelopeState, (int)offset, count);

                    if (_envelopeState.State == AdsrEnvelopeState.Completed)
                    {
                        StopProducingSound();
                    }


                }
                else
                {

                    // fill the buffer with silence / non-samples
                    _buffer.Clear();
                }
            }
            else
            {
                _buffer.Clear();
           
            }



        }

        private void StopProducingSound()
        {
            _isGeneratingSound = false;
            //_currentNote = null;
        }


        public void Trigger(Note currentNote)
        {
            //System.Diagnostics.Debug.WriteLine("Oscillator.Trigger");

            if (_parameters.IsEnabled)
            {
                _waveformState = _parameters.Waveform.GetNewState();

                _currentNote = currentNote;
                _isTriggered = true;
                _isGeneratingSound = true;
                //_currentWaveform = 


                // TEMP!!
                _envelopeState.State = AdsrEnvelopeState.Attack;
            }

        }

        public void Release()
        {
            //throw new NotImplementedException();
            //_currentNote = null;
            _isTriggered = false;

            // Note: to not set _isGeneratingSound to false here, as we have to take envelope into account

            // TEMP!!
            _envelopeState.State = AdsrEnvelopeState.Release;
            
        }


        public SoundBuffer<MonoSample> Buffer
        {
            get { return _buffer; }
        }

        public OscillatorParameters Parameters
        {
            get { return _parameters; }
        }

        //public IFilter Filter
        //{
        //    get { return _filter; }
        //    set { _filter = value; }
        //}

        //public bool IsTriggered
        //{
        //    get { return _isTriggered; }
        //}

        public bool IsGeneratingSound
        {
            get { return _isGeneratingSound; }
        }

    }
}
