﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using PeteBrown.SilverlightSoundTest.Synth.Wave;
using System.ComponentModel;
using PeteBrown.SilverlightSoundTest.Synth.Envelopes;

namespace PeteBrown.SilverlightSoundTest.Synth.Oscillators
{
    // Defines the shared parameters for all instances of an oscillator. In a simple analog 
    // synth, you have voices * NumOscillators physical oscillators. If the panel shows 2 
    // oscillators and the synth is capable of 8 voice polyphony, you have 16 oscillators

    public class OscillatorParameters: INotifyPropertyChanged
    {
        private string _title = "Oscillator";                   // For UI Convenience. should probably be in a view model, but I don't care :)

        private IWaveform _waveform;                                        // the waveform this oscillator uses to produce sound
        private Transpose _transpose = new Transpose(0, 0);                 // if we're transposing the note for oscillator
        private FrequencyOffset _detune = new FrequencyOffset(-3, 3, 0);    // for detuning the oscillator. value is frequency

        private Pan _pan = new Pan(Pan.Center);                             // Panning (Left, Center, Right etc.)

        private Polarity _polarity = Polarity.Positive;
        private Percent _phaseOffsetPercent = new Percent(0);

        private EnvelopeParameters _amplitudeEnvelopeParameters = new EnvelopeParameters();

        private bool _isEnabled = true;                                     // true if this oscillator is enabled

        public IWaveform Waveform
        {
            get { return _waveform; }
            set { _waveform = value; NotifyChanged("Waveform"); }
        }

        public Pan Pan
        {
            get { return _pan; }
            set { _pan = value; }
        }

        public Transpose Transpose
        {
            get { return _transpose; }
            set { _transpose = value; }
        }

        public FrequencyOffset Detune
        {
            get { return _detune; }
            set { _detune = value; }
        }

        public Polarity Polarity
        {
            get { return _polarity; }
            set { _polarity = value; NotifyChanged("Polarity"); }
        }

        public Percent PhaseOffset
        {
            get { return _phaseOffsetPercent; }
            set { _phaseOffsetPercent = value; NotifyChanged("PhaseOffset"); }
        }

        public EnvelopeParameters AmplitudeEnvelopeParameters
        {
            get { return _amplitudeEnvelopeParameters; }
            set { _amplitudeEnvelopeParameters = value; }
        }
        
        
        public bool IsEnabled
        {
            get { return _isEnabled; }
            set { _isEnabled = value; NotifyChanged("IsEnabled"); }
        }


        public string Title
        {
            get { return _title; }
            set { _title = value; NotifyChanged("Title"); }
        }



        #region INotifyPropertyChanged Members

        private void NotifyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }


        public event PropertyChangedEventHandler PropertyChanged;


        #endregion



    }
}
