﻿using System;
using System.Linq;
using System.Linq.Expressions;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO;
using System.Runtime.InteropServices;

using PeteBrown.SilverlightSoundTest.SoundRenderer;
using PeteBrown.SilverlightSoundTest.Synth.Oscillators;
using PeteBrown.SilverlightSoundTest.Synth.Wave;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using PeteBrown.SilverlightSoundTest.Synth.Envelopes;
using System.Collections;
using System.ComponentModel;

namespace PeteBrown.SilverlightSoundTest.Synth
{
    public partial class Synthesizer : INotifyPropertyChanged
    {
        private MemoryStream _stream;
        private SampleRate _sampleRate = new SampleRate(44100);

        private List<SoundBuffer<MonoSample>> _oscillatorBuffers = new List<SoundBuffer<MonoSample>>();
        private ObservableCollection<OscillatorParameters> _oscillatorParameters = new ObservableCollection<OscillatorParameters>();
        private List<Voice> _voices = new List<Voice>();

        private double _volume = .5;     // 0 to 1, but can go from 0 to infinity since it is just a multiplier. My amp goes to 11!
        private Arpeggiator _arpeggiator;

        //private Dictionary<string, IWaveform> _waveforms = new Dictionary<string, IWaveform>();

        // TEMP!!
        //private AmplitudeEnvelope _ampEnvelope = new AmplitudeEnvelope();


        public const int BitsPerSample = 16;    // change this and you need to change code that writes the sample too
        public const int ChannelCount = 2;      // 2 for Stereo


        private int _oscillatorMonoBufferSampleCount = 128;   // size of the mono buffer per-oscillator


        public event EventHandler<BufferPlayedEventArgs> BufferPlayed;





        public Synthesizer(int maxVoices)
        {
            InitializeMediaStreamSource();

            // allocate voices
            for (int i = 0; i < maxVoices; i++)
            {
                _voices.Add(new Voice());
            }

            _arpeggiator = new Arpeggiator(this);

        }

        //// TEMP!!
        //public void TempTriggerNote(Note note)
        //{
        //    foreach (IOscillator osc in _oscillators)
        //    {
        //        if (osc.IsEnabled)
        //            osc.Trigger(note);
        //    }
        //}

        public Voice TriggerNote(Note note)
        {
            System.Diagnostics.Debug.WriteLine("TriggerNote");
            // allocate a voice and return an instance of it

            foreach (Voice v in _voices)
            {
                if (!v.IsInUse)
                {
                    v.Trigger(note);
                    return v;
                }
            }

            // no available voices, time to steal from the oldest one
            // TODO!
            return null;
        }





        public void Output()
        {
            StereoSample peakValues = new StereoSample();

            //System.Diagnostics.Debug.WriteLine("Start Synthesizer Output " + DateTime.Now.Millisecond);

            // TODO: Optimize this to a single pass if at all possible

            int numActiveOscillators = 0;

            // output pass
            foreach (Voice v in _voices)
            {
                if (v.IsInUse)
                {
                    // output all oscillators for this voice
                    foreach (IOscillator osc in v.Oscillators)
                    {
                        if (osc.Parameters.IsEnabled)
                        {
                            numActiveOscillators++;
                            osc.Output(0, (int)_oscillatorMonoBufferSampleCount);
                        }
                    }

                }
            }


            // mixing pass

            //_stream.Seek(0, SeekOrigin.Begin);

            for (int i = 0; i < _oscillatorMonoBufferSampleCount; i++)
            {
                StereoSample sample = new StereoSample();

                // output pass
                foreach (Voice v in _voices)
                {
                    int voiceLeft = 0;
                    int voiceRight = 0;

                    if (v.IsInUse)
                    {
                        foreach (IOscillator osc in v.Oscillators)
                        {
                            if (osc.Parameters.IsEnabled)
                            {
                                if (osc.Buffer[i] != null)
                                {
                                    StereoSample mixed = MixStereo(osc.Buffer[i], osc.Parameters.Pan);

                                    //sample.Left += (int)((mixed.Left * Volume) * 2 / numActiveOscillators);
                                    //sample.Right += (int)((mixed.Right * Volume) * 2 / numActiveOscillators);

                                    // TEMP HARDCODING of divide by 3 for 3 max oscillators per voice
                                    voiceLeft += (int)(mixed.Left / 3);
                                    voiceRight += (int)(mixed.Right / 3);
                                }
                            }


                        }

                        //sample.Left += (int)((double)voiceLeft / _voices.Count);
                        //sample.Right += (int)((double)voiceRight / _voices.Count);
                        sample.Left += (int)(voiceLeft / 2);
                        sample.Right += (int)(voiceRight / 2);

                    }
                }

                sample.AdjustForVolume(Volume);

                //// constrain peaks
                //if (sample.Left >= short.MaxValue * .6 || sample.Left <= short.MinValue * .6 ||
                //    sample.Right >= short.MaxValue * .6 || sample.Right <= short.MinValue * .6)
                //{
                //    sample.Left *
                //}

                
                // set peak values for the buffer played event
                if (sample.Left > peakValues.Left)
                    peakValues.Left = sample.Left;

                if (sample.Right > peakValues.Right)
                    peakValues.Right = sample.Right;


                _stream.Write(sample.GetDownSampledBytes(), 0, sample.GetDownSampledByteCount());

            }


            foreach (Voice v in _voices)
            {
                // voice cycling deallocates the voice when no more output
                v.Cycle();
            }

            try
            {
                // this assumes we have a small buffer and are playing pretty much real-time
                if (BufferPlayed != null)
                    BufferPlayed(this, new BufferPlayedEventArgs(peakValues));
            }
            catch (Exception ex)
            {
            }


            //System.Diagnostics.Debug.WriteLine("Completed Synthesizer Output " + DateTime.Now.Millisecond);
        }


        private StereoSample MixStereo(MonoSample sample, Pan pan)
        {
            int left = 0;
            int right = 0;

            // left-leaning
            if (pan.Position < 0)
            {
                left = sample.Value;
                right = (int)(sample.Value - ((-1) * sample.Value * pan.Position));
            }

            // right-leaning
            else if (pan.Position > 0)
            {
                left = (int)(sample.Value - sample.Value * pan.Position);
                right = sample.Value;
            }

            // center. unlikely due to floating point inexactness
            else
            {
                left = sample.Value;
                right = sample.Value;
            }

            return new StereoSample(left, right);
        }


        // creates an oscillator def (and returns it), and creates an instance of the oscillator for each voice
        public OscillatorParameters AddOscillator()
        {
            OscillatorParameters parameters = new OscillatorParameters();

            EnvelopeParameters envelopeParameters = new EnvelopeParameters();
            envelopeParameters.Initialize(_sampleRate);
            parameters.AmplitudeEnvelopeParameters = envelopeParameters;

            _oscillatorParameters.Add(parameters);

            
            foreach (Voice v in _voices)
            {
                SoundBuffer<MonoSample> buffer = new SoundBuffer<MonoSample>(_oscillatorMonoBufferSampleCount);
                _oscillatorBuffers.Add(buffer);

                IOscillator osc = new Oscillator();

                // TEMP

                osc.Initialize(buffer, parameters);
                v.Oscillators.Add(osc);
            }

            return parameters;
        }

        public OscillatorParameters GetOscillatorParameters(int index)
        {
            return _oscillatorParameters[index];
        }


        //public void AddWaveform(string key, IWaveform waveform)
        //{
        //    waveform.Initialize(_sampleRate);

        //    _waveforms.Add(key, waveform);
        //}


        public IWaveform CreateWhiteNoiseWaveform()
        {
            IWaveform waveform = new WhiteNoiseWaveform();
            waveform.Initialize(_sampleRate);

            return waveform;
        }

        public IWaveform CreateSineWaveform()
        {
            IWaveform waveform = new SineWaveform();
            waveform.Initialize(_sampleRate);

            return waveform;
        }

        public IWaveform CreateSquareWaveform()
        {
            IWaveform waveform = new SquareWaveform();
            waveform.Initialize(_sampleRate);

            return waveform;
        }

        public IWaveform CreateSawtoothWaveform()
        {
            IWaveform waveform = new SawtoothWaveform();
            waveform.Initialize(_sampleRate);

            return waveform;
        }

        public IWaveform CreateFatSawWaveform()
        {
            IWaveform waveform = new FatSawWaveform();
            waveform.Initialize(_sampleRate);

            return waveform;
        }


        public IWaveform CreateTriangleWaveform()
        {
            IWaveform waveform = new TriangleWaveform();
            waveform.Initialize(_sampleRate);

            return waveform;
        }

        //public IWaveform GetWaveform(string key)
        //{
        //    return _waveforms[key];
        //}

        //public ObservableCollection<IOscillator> Oscillators
        //{
        //    get { return _oscillators; }
        //}


        public Stream AudioStream
        {
            get { return _stream; }
        }

        public uint ByteRate
        {
            get { return (uint)_sampleRate.SamplesPerSecond * ChannelCount * BitsPerSample / 8; }
        }


        public Arpeggiator Arpeggiator
        {
            get { return _arpeggiator; }
        }

        public double Volume
        {
            get { return _volume; }
            set { _volume = value; NotifyChanged("Volume"); }
        }


        #region INotifyPropertyChanged Members

        private void NotifyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }


        public event PropertyChangedEventHandler PropertyChanged;


        #endregion


    }
}
