﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Diagnostics;

namespace PeteBrown.SilverlightSoundTest.Synth
{
    public class Transpose : INotifyPropertyChanged
    {
        private int _octave;
        private int _semiTone;

        public const int MinSemiTone = -11;
        public const int MaxSemiTone = 11;
        public const int NoSemiTone = 0;

        public const int MinOctave = -1;
        public const int MaxOctave = 1;
        public const int NoOctave = 0;

        public Transpose(int octave, int semiTone)
        {
            _octave = ScrubOctave(octave);
            _semiTone = ScrubSemiTone(semiTone);
        }

        public int Octave
        {
            get { return _octave; }
            set { _octave = ScrubOctave(value); NotifyChanged("Octave");  }
        }

        public int SemiTone
        {
            get { return _semiTone; }
            set { _semiTone = ScrubSemiTone(value); NotifyChanged("SemiTone"); }
        }


        private int ScrubOctave(int octave)
        {
            if (octave < MinOctave) return MinOctave;
            if (octave > MaxOctave) return MaxOctave;

            return octave;
        }

        private int ScrubSemiTone(int semiTone)
        {
            if (semiTone < MinSemiTone) return MinSemiTone;
            if (semiTone > MaxSemiTone) return MaxSemiTone;

            return semiTone;
        }

        [DebuggerStepThrough()]
        public double GetFrequency(Note baseNote)
        {
            // TODO: Check and account for going above or below note range

            int noteNumber = baseNote.Number + _octave * 12 + _semiTone;

            return Note.GetFrequency(noteNumber);
               
        }


        #region INotifyPropertyChanged Members

        private void NotifyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }


        public event PropertyChangedEventHandler PropertyChanged;


        #endregion
    }
}
