﻿#region Copyright
// Substantial portions adapted from Leslie Sanford's code. Copyright below

/* Copyright (c) 2007 Leslie Sanford
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to 
 * deal in the Software without restriction, including without limitation the 
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
 * sell copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
#endregion

using System;


namespace PeteBrown.SilverlightSoundTest.Synth.Utilities
{
    public class PowerOfTwoTable
    {
        private const int TableSize = 8192;
        private static readonly float[] _table = new float[TableSize];


        static PowerOfTwoTable()
        {
            // Calculate phase increment.
            float increment = 1.0f / TableSize;

            // Phase accumulator.
            float accumulator = 0;

            // Initialize lookup table.
            for (int i = 0; i < TableSize; i++)
            {
                _table[i] = (float)Math.Pow(2, accumulator);
                accumulator += increment;
            }
        }

        public static float GetPower(float exponent)
        {
            float result;

            if (exponent >= 0)
            {
                // If exponent is positive.

                // Get whole part of exponent.
                int whole = (int)exponent;

                // Get fractional part of exponent.
                float fractional = exponent - whole;

                // Get index into lookup table. Some form of interpolation
                // would help here, but with a large enough lookup table, the
                // results seem acceptable.
                int index = (int)(TableSize * fractional);

                // Calculate final result. 
                result = _table[index] * (1 << whole);
            }
            else
            {
                // Else exponent is negative.

                // Get whole part of exponent.
                int whole = (int)-exponent;

                // Get fractional part of exponent.
                float fractional = -exponent - whole;

                // Get index into lookup table. Some form of interpolation
                // would help here, but with a large enough lookup table, the
                // results seem acceptable.
                int index = (int)(TableSize * fractional);

                // Calculate final result - divide 1 by result since exponent is negative. 
                result = 1.0f / (_table[index] * (1 << whole));
            }

            return result;
        }


    }
}
