﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using PeteBrown.SilverlightSoundTest.Synth.Oscillators;
using PeteBrown.SilverlightSoundTest.Synth.Envelopes;
using System.Collections.Generic;

namespace PeteBrown.SilverlightSoundTest.Synth
{
    public class Voice
    {
        private List<IOscillator> _oscillators = new List<IOscillator>();
        
        public bool IsInUse { get; set; }
        private Note _note;

        //private SoundBuffer<StereoSample> _buffer;
        //private SoundBuffer<MonoSample> _oscillatorBuffer;

        public Voice()
        {
            IsInUse = false;
        }

        public void Trigger(Note note)
        {
            //System.Diagnostics.Debug.WriteLine("Voice.Trigger");

            IsInUse = true;
            _note = note;

            foreach (IOscillator osc in _oscillators)
            {
                osc.Trigger(_note);
            }
        }

        public void Release()
        {
            // todo: must go through release cycle before deallocating

            // TEMP!!
            //IsInUse = false;
            //_note = null;
            foreach (IOscillator osc in _oscillators)
            {
                osc.Release();
            }
        }


        // called once per buffering cycle. Would be better to have this more finely 
        // grained, but there's no real harm to calling it less frequently unless we're
        // super tight on voices.
        public void Cycle()
        {
            // check all oscillators to see if they're generating sound

            foreach (IOscillator osc in _oscillators)
            {
                if (osc.IsGeneratingSound)
                    return;
            }

            // no oscillators are generating sound
            Deallocate();
        }


        private void Deallocate()
        {
            IsInUse = false;
            _note = null;

        }


        public List<IOscillator> Oscillators
        {
            get { return _oscillators; }
        }

    }
}
