﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using PeteBrown.SilverlightSoundTest.Synth.Utilities;

namespace PeteBrown.SilverlightSoundTest.Synth.Wave
{
    public class FatSawWaveform : IWaveform
    {
        private SampleRate _sampleRate;
        private const int MaxWaveforms = 6;

        //private double[] _detune = new double[MaxWaveforms] {  0,
        //                                                       -1.25, -1.00, -0.50, -0.25, -.10, 
        //                                                       .10, .25, .50, 1.00, 1.25 };
        //private double[] _detune;

        #region IWaveform Members

        public void Initialize(SampleRate rate)
        {
            _sampleRate = rate;

            //_detune = new double[MaxWaveforms]; 

        }

        public MonoSample GetSample(double frequency, WaveformState state)
        {
            double accumulatedAmplitude = 0;

            // HACK: weasle out of problems with changing waveforms mid-note
            // note, I don't handle having stale values in the other slots
            // just yet. need to, though, as it contributes to noise and artifacts.
            if (state.Phase.Length < MaxWaveforms)
            {
                state.Phase = new double[MaxWaveforms];
            }

            // calculate detune based on provided frequency. not much science here, just trying
            // to get something that is somewhat reminiscent of the roland SuperSaw. It's definitely
            // missing the lower-end oomph, though. Not sure what to do there yet other than perhaps
            // model an LFO mixed in with this waveform. Real SuperSaw uses 7 detuned waveforms, but 
            // no one has published info on how much detuning.
            // Some history http://everything2.com/e2node/Roland%2520JP-8000

            // the differing offsets is to help minimize phasing. should change to cents rather than 
            // fractional amount. 12 cents would be a good amount. would also be good to make this a 
            // "width" parameter specific to this waveform. Need to consider how to surface waveform-
            // specific parameters to UI and patch storage
            double[] detune = CalculateSpread(frequency);

            //double spread = 15;


            for (int i = 0; i < MaxWaveforms; i++)
            {
                double phase = CalculatePhaseWithOffset(state.PhaseOffset, state.Phase[i]);
                double sample = (state.MaxAmplitude.Value - (state.MaxAmplitude.Value / Math.PI) * phase) * (int)state.Polarity;

                //if (i % 2 != 0)
                //    sample *= -1;

                accumulatedAmplitude += sample;

                state.Phase[i] = CheckPhase(state.Phase[i] + ((MathUtility.TwoPI * detune[i]) / _sampleRate.SamplesPerSecond));

                //accumulatedAmplitude *= .75;    // warning, magic number at work
            
            }

            accumulatedAmplitude /= MaxWaveforms;

            #region Old
            //double primaryAmplitude = (primaryMaxAmplitude - (primaryMaxAmplitude / Math.PI) * _phase1 * (short)polarity);

            //double amplitude2 = (maxAmplitude - (maxAmplitude / Math.PI) * _phase2 * (short)polarity);
            //double amplitude3 = (maxAmplitude - (maxAmplitude / Math.PI) * _phase3 * (short)polarity);
            //double amplitude4 = (maxAmplitude - (maxAmplitude / Math.PI) * _phase4 * (short)polarity);
            //double amplitude5 = (maxAmplitude - (maxAmplitude / Math.PI) * _phase5 * (short)polarity);
            //double amplitude6 = (maxAmplitude - (maxAmplitude / Math.PI) * _phase6 * (short)polarity);
            //double amplitude7 = (maxAmplitude - (maxAmplitude / Math.PI) * _phase7 * (short)polarity);

            //short amplitude = (short)((primaryAmplitude + amplitude2 + amplitude3 + amplitude4 + amplitude5 + amplitude6 + amplitude7) / 7);


            //_phase1 = _phase1 + ((2 * Math.PI * (frequency)) / _sampleRate.SamplesPerSecond);

            //_phase2 = _phase2 + ((2 * Math.PI * (frequency + .50)) / _sampleRate.SamplesPerSecond);
            //_phase3 = _phase3 + ((2 * Math.PI * (frequency - .55)) / _sampleRate.SamplesPerSecond);

            //_phase4 = _phase4 + ((2 * Math.PI * (frequency + .75)) / _sampleRate.SamplesPerSecond);
            //_phase5 = _phase5 + ((2 * Math.PI * (frequency - .70)) / _sampleRate.SamplesPerSecond);

            //_phase6 = _phase6 + ((2 * Math.PI * (frequency + 0.90)) / _sampleRate.SamplesPerSecond);
            //_phase7 = _phase7 + ((2 * Math.PI * (frequency - 1.00)) / _sampleRate.SamplesPerSecond);


            //_phase1 = CheckPhase(_phase1);
            //_phase2 = CheckPhase(_phase2);
            //_phase3 = CheckPhase(_phase3);
            //_phase4 = CheckPhase(_phase4);
            //_phase5 = CheckPhase(_phase5);
            //_phase6 = CheckPhase(_phase6);
            //_phase7 = CheckPhase(_phase7);
            #endregion

            return new MonoSample(accumulatedAmplitude);
        }


        private double CalculatePhaseWithOffset(Percent phaseOffsetPercent, double rawPhase)
        {
            return CheckPhase(rawPhase + phaseOffsetPercent.Value * MathUtility.TwoPI);
        }


        private double CheckPhase(double phase)
        {
            if (phase > MathUtility.TwoPI)
                return phase - MathUtility.TwoPI;
            else
                return phase;
        }


        // TODO: Make this algorithm apply to all waveforms and use a spread parameter
        private double[] CalculateSpread(double baseFrequency)
        {
            double[] detune = new double[MaxWaveforms];

            //double spread = 15;

            // TODO: Make this algorithm apply to all waveforms and use a spread parameter
            detune[0] = baseFrequency;
            detune[1] = baseFrequency - 0.10;          // slight detune
            detune[2] = baseFrequency * 4 + 1.06;      // two octaves up
            detune[3] = baseFrequency * 2 - 1.05;      // one octave up
            detune[4] = baseFrequency / 2 + 1.11;      // one octave down
            detune[5] = baseFrequency / 2 - 1.18;      // one octave down

            return detune;
        }


        public WaveformState GetNewState()
        {
            return new WaveformState(MaxWaveforms);
        }

        public int GetSampleCount(double frequency)
        {
            double lowestFrequency = frequency;
            double[] detune = CalculateSpread(frequency);

            for (int i = 0; i < detune.Length; i++)
            {
                if (detune[i] < lowestFrequency)
                    lowestFrequency = detune[i];
            }

            return (int)Math.Round(_sampleRate.SamplesPerSecond / lowestFrequency, 0);
        }


        #endregion
    }
}
