﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using PeteBrown.SilverlightSoundTest.Synth.Utilities;

namespace PeteBrown.SilverlightSoundTest.Synth.Wave
{
    public class SawtoothWaveform : IWaveform
    {
        private SampleRate _sampleRate;

        #region IWaveform Members

        public void Initialize(SampleRate rate)
        {
            _sampleRate = rate;
        }

        // ref http://en.wikibooks.org/wiki/Sound_Synthesis_Theory/Oscillators_and_Wavetables
        public MonoSample GetSample(double frequency, WaveformState state)
        {
            double amplitude;
            double phase = CalculatePhaseWithOffset(state.PhaseOffset, state.Phase[0]);

            amplitude = (state.MaxAmplitude.Value - (state.MaxAmplitude.Value / Math.PI) * phase) * (int)state.Polarity;

            state.Phase[0] = CheckPhase(state.Phase[0] + (MathUtility.TwoPI * frequency / _sampleRate.SamplesPerSecond));

            return new MonoSample(amplitude);
        }


        private double CalculatePhaseWithOffset(Percent phaseOffsetPercent, double rawPhase)
        {
            return CheckPhase(rawPhase + phaseOffsetPercent.Value * MathUtility.TwoPI);
        }


        private double CheckPhase(double phase)
        {
            if (phase > MathUtility.TwoPI)
                return phase - MathUtility.TwoPI;
            else
                return phase;
        }
        
        public WaveformState GetNewState()
        {
            return new WaveformState(1);
        }

        // TODO: Have this returning a meaningful number
        public int GetSampleCount(double frequency)
        {
            return (int)Math.Round(_sampleRate.SamplesPerSecond / frequency, 0);
        }

        #endregion
    }
}
