﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using PeteBrown.SilverlightSoundTest.Synth.Utilities;

namespace PeteBrown.SilverlightSoundTest.Synth.Wave
{
    public class SineWaveform : IWaveform
    {
        private SampleRate _sampleRate;
        private const int MaxWaveforms = 1;

        #region IWaveform Members

        public void Initialize(SampleRate rate)
        {
            _sampleRate = rate;
        }

        public MonoSample GetSample(double frequency, WaveformState state)
        {
            double accumulatedAmplitude = 0;

            double[] detune = new double[MaxWaveforms];

            ////double spread = 15;

            //// TODO: Make this algorithm apply to all waveforms and use a spread parameter
            detune[0] = frequency;
            //detune[1] = frequency - 0.10;          // slight detune
            //detune[2] = frequency * 4 + 1.06;      // two octaves up
            //detune[3] = frequency * 2 - 1.05;      // one octave up
            //detune[4] = frequency / 2 + 1.11;      // one octave down
            //detune[5] = frequency / 2 - 1.18;      // one octave down

            //for (int i = 0; i < MaxWaveforms; i++)
            //{
            int i = 0;

            double phase = CalculatePhaseWithOffset(state.PhaseOffset, state.Phase[i]);
            double sample = Math.Sin(phase) * state.MaxAmplitude.Value * (int)state.Polarity;

                accumulatedAmplitude += sample;

                state.Phase[i] = CheckPhase(state.Phase[i] + ((MathUtility.TwoPI * detune[i]) / _sampleRate.SamplesPerSecond));

                //accumulatedAmplitude *= .75;    // warning, magic number at work

            //}

            //accumulatedAmplitude /= MaxWaveforms;   // TEMP

            return new MonoSample(accumulatedAmplitude);

        }

        private double CalculatePhaseWithOffset(Percent phaseOffsetPercent, double rawPhase)
        {
            return CheckPhase(rawPhase + phaseOffsetPercent.Value * MathUtility.TwoPI);
        }


        private double CheckPhase(double phase)
        {
            if (phase > MathUtility.TwoPI)
                return phase - MathUtility.TwoPI;
            else
                return phase;
        }


        public WaveformState GetNewState()
        {
            return new WaveformState(MaxWaveforms);
        }

        // TODO: Have this returning a meaningful number
        public int GetSampleCount(double frequency)
        {
            return (int)Math.Round(_sampleRate.SamplesPerSecond / frequency, 0);
        }



        #endregion
    }
}
