﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using PeteBrown.SilverlightSoundTest.Synth.Utilities;

namespace PeteBrown.SilverlightSoundTest.Synth.Wave
{
    public class TriangleWaveform : IWaveform
    {
        private SampleRate _sampleRate;

        #region IWaveform Members

        public void Initialize(SampleRate rate)
        {
            _sampleRate = rate;
        }

        // ref http://en.wikibooks.org/wiki/Sound_Synthesis_Theory
        // ref http://en.wikibooks.org/wiki/Sound_Synthesis_Theory/Oscillators_and_Wavetables
        // ref http://en.wikipedia.org/wiki/Triangle_wave
        public MonoSample GetSample(double frequency, WaveformState state)
        {
            double amplitude;

            //double amplitude = Math.Acos(Math.Sin(state.Phase[0])) * state.MaxAmplitude.Value * (int)state.Polarity;

            double phase = CalculatePhaseWithOffset(state.PhaseOffset, state.Phase[0]);

            if (phase < Math.PI)
                amplitude = ((-1.0 * state.MaxAmplitude.Value) + (2.0 * state.MaxAmplitude.Value / Math.PI) * phase) * (int)state.Polarity;
            else
                amplitude = ((3.0 * state.MaxAmplitude.Value) - (2.0 * state.MaxAmplitude.Value / Math.PI) * phase) * (int)state.Polarity;

            state.Phase[0] = CheckPhase(state.Phase[0] + ((MathUtility.TwoPI * frequency) / _sampleRate.SamplesPerSecond));

            return new MonoSample(amplitude);
        }

        private double CalculatePhaseWithOffset(Percent phaseOffsetPercent, double rawPhase)
        {
            return CheckPhase(rawPhase + phaseOffsetPercent.Value * MathUtility.TwoPI);
        }

        private double CheckPhase(double phase)
        {
            if (phase > MathUtility.TwoPI)
                return phase - MathUtility.TwoPI;
            else
                return phase;
        }


        public WaveformState GetNewState()
        {
            WaveformState state = new WaveformState(1);

            return state;
        }

        // TODO: Have this returning a meaningful number
        public int GetSampleCount(double frequency)
        {
            return (int)Math.Round(_sampleRate.SamplesPerSecond / frequency, 0);
        }


        #endregion    
    }
}
