﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using PeteBrown.SilverlightSoundTest.Synth;
using System.Windows.Controls.Primitives;

namespace PeteBrown.SilverlightSoundTest.UI
{
    public partial class Keyboard : UserControl
    {
        private Synthesizer _synth;

        public Keyboard()
        {
            InitializeComponent();
            Loaded += new RoutedEventHandler(Keyboard_Loaded);
        }

        void Keyboard_Loaded(object sender, RoutedEventArgs e)
        {
            BuildKeyboard();
        }

        public Synthesizer Synth
        {
            get { return _synth; }
            set { _synth = value; }
        }


        private void BuildKeyboard()
        {
            Canvas c = new Canvas();

            int normalKeyWidth = 20;
            int normalKeyHeight = 100;

            int sharpKeyWidth = 15;
            int sharpKeyHeight = 60;

            int sharpKeyLeftOffset = normalKeyWidth - sharpKeyWidth / 2;

            int currentLeft = 0;


            LayoutRoot.Height = normalKeyHeight;

            for (int i = 2; i < 8; i++)
            {
                c.Children.Add(CreateNormalKeyButton(new Note(Notes.C, i), normalKeyWidth, normalKeyHeight, currentLeft, 0));
                c.Children.Add(CreateSharpKeyButton(new Note(Notes.CSharp, i), sharpKeyWidth, sharpKeyHeight, currentLeft + sharpKeyLeftOffset, 0));
                currentLeft += normalKeyWidth;

                c.Children.Add(CreateNormalKeyButton(new Note(Notes.D, i), normalKeyWidth, normalKeyHeight, currentLeft, 0));
                c.Children.Add(CreateSharpKeyButton(new Note(Notes.DSharp, i), sharpKeyWidth, sharpKeyHeight, currentLeft + sharpKeyLeftOffset, 0));
                currentLeft += normalKeyWidth;


                c.Children.Add(CreateNormalKeyButton(new Note(Notes.E, i), normalKeyWidth, normalKeyHeight, currentLeft, 0));
                currentLeft += normalKeyWidth;

                c.Children.Add(CreateNormalKeyButton(new Note(Notes.F, i), normalKeyWidth, normalKeyHeight, currentLeft, 0));
                c.Children.Add(CreateSharpKeyButton(new Note(Notes.FSharp, i), sharpKeyWidth, sharpKeyHeight, currentLeft + sharpKeyLeftOffset, 0));
                currentLeft += normalKeyWidth;


                c.Children.Add(CreateNormalKeyButton(new Note(Notes.G, i), normalKeyWidth, normalKeyHeight, currentLeft, 0));
                c.Children.Add(CreateSharpKeyButton(new Note(Notes.GSharp, i), sharpKeyWidth, sharpKeyHeight, currentLeft + sharpKeyLeftOffset, 0));
                currentLeft += normalKeyWidth;

                c.Children.Add(CreateNormalKeyButton(new Note(Notes.A, i), normalKeyWidth, normalKeyHeight, currentLeft, 0));
                c.Children.Add(CreateSharpKeyButton(new Note(Notes.ASharp, i), sharpKeyWidth, sharpKeyHeight, currentLeft + sharpKeyLeftOffset, 0));
                currentLeft += normalKeyWidth;


                c.Children.Add(CreateNormalKeyButton(new Note(Notes.B, i), normalKeyWidth, normalKeyHeight, currentLeft, 0));
                currentLeft += normalKeyWidth;

            }

            LayoutRoot.Children.Add(c);

            LayoutRoot.Width = currentLeft;
        }

        private ButtonBase CreateNormalKeyButton(Note note, int width, int height, int leftPosition, int topPosition)
        {
            KeyboardKeyButton button = new KeyboardKeyButton();
            button.Content = note.ToString();
            button.DataContext = note;

            button.Style = (Style)Application.Current.Resources["WhiteKeyboardKey"];

            button.Width = width;
            button.Height = height;

            button.SetValue(Canvas.TopProperty, (double)topPosition);
            button.SetValue(Canvas.LeftProperty, (double)leftPosition);
            button.SetValue(Canvas.ZIndexProperty, (int)0);


            button.Checked += new RoutedEventHandler(button_Checked);
            button.Unchecked += new RoutedEventHandler(button_Unchecked);

            return button;
        }

        private ButtonBase CreateSharpKeyButton(Note note, int width, int height, int leftPosition, int topPosition)
        {
            KeyboardKeyButton button = new KeyboardKeyButton();
            button.Content = note.ToString();
            button.DataContext = note;

            button.Style = (Style)Application.Current.Resources["BlackKeyboardKey"];

            button.Width = width;
            button.Height = height;

            button.SetValue(Canvas.TopProperty, (double)topPosition);
            button.SetValue(Canvas.LeftProperty, (double)leftPosition);
            button.SetValue(Canvas.ZIndexProperty, (int)1);


            button.Checked += new RoutedEventHandler(button_Checked);
            button.Unchecked += new RoutedEventHandler(button_Unchecked);

            return button;
        }




        void button_Unchecked(object sender, RoutedEventArgs e)
        {
            KeyboardKeyButton button = (KeyboardKeyButton)sender;
            button.VoiceInUse.Release();
        }

        void button_Checked(object sender, RoutedEventArgs e)
        {
            KeyboardKeyButton button = (KeyboardKeyButton)sender;
            button.VoiceInUse = _synth.TriggerNote((Note)(button.DataContext));
        }


    }
}
