﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using PeteBrown.SilverlightSoundTest.Synth.Oscillators;
using PeteBrown.SilverlightSoundTest.Synth.Wave;
using PeteBrown.SilverlightSoundTest.Synth;

namespace PeteBrown.SilverlightSoundTest.UI
{
    public partial class OscillatorEditor : UserControl
    {
        private OscillatorParameters _oscillatorParameters;

        private Synthesizer _synth;

        public OscillatorEditor()
        {
            InitializeComponent();

            this.SineWaveform.Click += new RoutedEventHandler(SineWaveform_Click);
            this.SquareWaveform.Click += new RoutedEventHandler(SquareWaveform_Click);
            this.WhiteNoiseWaveform.Click += new RoutedEventHandler(NoiseWaveform_Click);
            this.TriangleWaveform.Click += new RoutedEventHandler(TriangleWaveform_Click);
            this.SawtoothWaveform.Click += new RoutedEventHandler(SawtoothWaveform_Click);
            this.FatSawWaveform.Click += new RoutedEventHandler(SuperSawWaveform_Click);

            this.PositivePolarity.Checked += new RoutedEventHandler(PositivePolarity_Checked);
            this.NegativePolarity.Checked += new RoutedEventHandler(NegativePolarity_Checked);


            this.PhaseOffsetSlider.ValueChanged += new RoutedPropertyChangedEventHandler<double>(PhaseOffsetSlider_ValueChanged);

            //this.PositivePolarity.Unchecked += new RoutedEventHandler(PositivePolarity_Unchecked);
            //this.NegativePolarity.Unchecked += new RoutedEventHandler(NegativePolarity_Unchecked);
        }

        void PhaseOffsetSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            UpdateWaveformVisualization();
        }

        //void NegativePolarity_Unchecked(object sender, RoutedEventArgs e)
        //{
        //    throw new NotImplementedException();
        //}

        //void PositivePolarity_Unchecked(object sender, RoutedEventArgs e)
        //{
        //    throw new NotImplementedException();
        //}

        void PositivePolarity_Checked(object sender, RoutedEventArgs e)
        {
            _oscillatorParameters.Polarity = Polarity.Positive;
            UpdateWaveformVisualization();
        }

        void NegativePolarity_Checked(object sender, RoutedEventArgs e)
        {
            _oscillatorParameters.Polarity = Polarity.Negative;
            UpdateWaveformVisualization();
        }

        public Synthesizer Synth
        {
            get { return _synth; }
            set { _synth = value; }
        }

        public OscillatorParameters OscillatorParameters
        {
            get { return _oscillatorParameters; }
            set { _oscillatorParameters = value; SyncUI(); DataContext = _oscillatorParameters; }
        }

        void SuperSawWaveform_Click(object sender, RoutedEventArgs e)
        {
            _oscillatorParameters.Waveform = _synth.CreateFatSawWaveform();
            UpdateWaveformVisualization();
        }

        void SawtoothWaveform_Click(object sender, RoutedEventArgs e)
        {
            _oscillatorParameters.Waveform = _synth.CreateSawtoothWaveform();
            UpdateWaveformVisualization();
        }

        void TriangleWaveform_Click(object sender, RoutedEventArgs e)
        {
            _oscillatorParameters.Waveform = _synth.CreateTriangleWaveform();
            UpdateWaveformVisualization();
        }

        void NoiseWaveform_Click(object sender, RoutedEventArgs e)
        {
            _oscillatorParameters.Waveform = _synth.CreateWhiteNoiseWaveform();
            UpdateWaveformVisualization();
        }

        void SquareWaveform_Click(object sender, RoutedEventArgs e)
        {
            _oscillatorParameters.Waveform = _synth.CreateSquareWaveform();
            UpdateWaveformVisualization();
        }

        void SineWaveform_Click(object sender, RoutedEventArgs e)
        {
            _oscillatorParameters.Waveform = _synth.CreateSineWaveform();
            UpdateWaveformVisualization();
        }

        void SyncUI()
        {
            if (_oscillatorParameters != null)
            {
                if (_oscillatorParameters.Waveform is SineWaveform)
                {
                    this.SineWaveform.IsChecked = true;
                }
                else if (_oscillatorParameters.Waveform is SquareWaveform)
                {
                    this.SquareWaveform.IsChecked = true;
                }
                else if (_oscillatorParameters.Waveform is WhiteNoiseWaveform)
                {
                    this.WhiteNoiseWaveform.IsChecked = true;
                }
                else if (_oscillatorParameters.Waveform is TriangleWaveform)
                {
                    this.TriangleWaveform.IsChecked = true;
                }
                else if (_oscillatorParameters.Waveform is SawtoothWaveform)
                {
                    this.SawtoothWaveform.IsChecked = true;
                }
                else if (_oscillatorParameters.Waveform is FatSawWaveform)
                {
                    this.FatSawWaveform.IsChecked = true;
                }


                if (_oscillatorParameters.Polarity == Polarity.Negative)
                    this.NegativePolarity.IsChecked = true;
                else
                    this.PositivePolarity.IsChecked = true;


                // binding to MinOffset / MaxOffset causes an error. Not sure why
                DetuneSlider.Minimum = _oscillatorParameters.Detune.MinOffset;
                DetuneSlider.Maximum = _oscillatorParameters.Detune.MaxOffset;

                OctaveSlider.Minimum = Transpose.MinOctave;
                OctaveSlider.Maximum = Transpose.MaxOctave;

                SemiToneSlider.Minimum = Transpose.MinSemiTone;
                SemiToneSlider.Maximum = Transpose.MaxSemiTone;

                PanSlider.Minimum = Pan.Left;
                PanSlider.Maximum = Pan.Right;

                UpdateWaveformVisualization();
            }

        }


        private void UpdateWaveformVisualization()
        {
            SoundBuffer<MonoSample> buffer;


            // Major hack. need to design with this in mind next time around and/or refactor now

            IOscillator osc = new Oscillator();
            osc.Initialize(null, _oscillatorParameters);
            buffer = osc.OutputOneCompleteCycle(new Note(Notes.C, 4));

            // here endeth the hack

            // Next, Plot the points on an image.

            int imageWidth = (int)WaveformVisualization.Width;
            int imageHeight = (int)WaveformVisualization.Height;

            double xfactor = (double)buffer.Length / imageWidth;

            WaveformVisualization.Children.Clear();


            Polyline line = new Polyline();
            //line.Stroke = new SolidColorBrush(Color.FromArgb(255, 64, 64, 64));
            line.Stroke = (Brush)Application.Current.Resources["LcdLineBrush"];
            line.StrokeThickness = 1;
            line.UseLayoutRounding = true;  // gives it more of an LCD look

            // origin
     //       line.Points.Add(new Point(0, imageHeight / 2));

            // waveform points

            line.Points.Add(new Point(0, 0));
            for (int x = 0; x < imageWidth; x++)
            {
                int sampleIndex = (int)Math.Round(x * xfactor, 0);
                //short sampleValue = buffer[sampleIndex].GetValueAs16Bits();
                int downsampled = (int)Math.Round(((double)buffer[sampleIndex].Value / int.MaxValue) * (imageHeight/2), 0) + imageHeight/2;

                int y = downsampled * -1 + imageHeight;

                if (y > imageHeight-1) y = imageHeight-1;
                if (y < 0) y = 0;

                // the image is indexed with 0,0 at the top left. The sample
                // has 0,0 as the middle of the left most vertical pixel line
                // with negative numbers below that and positive above

                //int y = (int)Math.Round((double)(-1 * sampleValue + short.MaxValue) / yfactor, 0);

                //bitmap[y * imageWidth + x] = BuildPgraColor(255, 0, 0, 255);

                line.Points.Add(new Point(x, y));

            }

            // replace the first point with the last point values
            if (line.Points.Count > 0)
            {
                line.Points[0] = new Point(0, line.Points[line.Points.Count - 1].Y);
            }


            // origin
   //         line.Points.Add(new Point(imageWidth-1, imageHeight / 2));


            WaveformVisualization.Children.Add(line);

        }


        //private int BuildPgraColor(byte red, byte green, byte blue, byte alpha)
        //{
        //    byte[] color = new byte[4];

        //    color[0] = blue;
        //    color[1] = green;
        //    color[2] = red;
        //    color[3] = alpha;

        //    return BitConverter.ToInt32(color, 0);
        //}



    }
}
